package com.huifu.jh.core;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.huifu.jh.core.model.*;
import com.huifu.jh.core.util.*;
import org.apache.commons.lang.StringUtils;
import org.w3c.dom.Document;

import com.huifu.jh.core.exception.GcbPayException;
import com.huifu.jh.core.message.FileResponseParser;
import com.huifu.jh.core.message.ParamStringResponseParser;
import com.huifu.jh.core.message.ResponseParser;
import com.huifu.jh.core.util.dto.GcbPayConstants;
import com.huifu.jh.core.util.dto.HttpResp;

/**
 * @author fang.wei
 * @Description 默认客户端
 * @createTime 2021年10月13日 16:35:00
 */
public class DefaultGcbPayClient implements GcbPayClient {	
    private static final String SIGN_MESSAGE_KEY = "signMsg";

    /**
     * 接口服务url
     */
    private String serverUrl;

    /**
     * 汇付国际公钥
     */
    private String gcbPublicKey;

    /**
     * 商户私钥
     */
    private String privateKey;

    /**
     * 商户配置
     */
    private Map<String, MerchantConfig> configMap = new HashMap<>();

    public DefaultGcbPayClient() {
    }

    public DefaultGcbPayClient(String serverUrl, String privateKey, String gcbPublicKey) {
        this.serverUrl = serverUrl;
        this.privateKey = privateKey;
        this.gcbPublicKey = gcbPublicKey;
    }

    public DefaultGcbPayClient(String serverUrl, List<MerchantConfig> configList, String gcbPublicKey) {
        this.serverUrl = serverUrl;
        if (configList != null) {
            for (MerchantConfig config : configList) {
                this.configMap.put(config.getMerchantId(), config);
            }
        }
    }

    @Override
    public <T extends GcbPayResponse> T execute(GcbPayRequest<T> request) throws Exception {
        if (request instanceof XmlRequest) {
            XmlRequest<T> xmlRequest = (XmlRequest<T>) request;
            return executeXmlRequest(xmlRequest);
        }

        ParamRequest<T> paramRequest = (ParamRequest<T>) request;
        return executeParamRequest(paramRequest);
    }

    /**
     * 执行form格式请求
     * @param request
     * @param <T>
     * @return
     * @throws Exception
     */
    private <T extends GcbPayResponse> T executeParamRequest(ParamRequest<T> request) throws Exception {
        Map<String, String> requestData = buildRequest(request);
        
        System.out.println("Request: " + requestData);
        
        HttpResp resp = HttpsClientUtils.postRequest(serverUrl, requestData, GcbPayConstants.ENCODING);        
        if (!resp.isSuccess()) {
            T response = request.getResponseClass().newInstance();
            response.setHttpStatus(resp.getHttpStatus());
            response.setResponseBody(resp.getResponseBody());
            return response;
        }

        T response = buildResponse(request, resp);
        return response;
    }

    /**
     * 构建响应
     * @param request
     * @param resp
     * @param <T>
     * @return
     * @throws Exception
     */
    private <T extends GcbPayResponse> T buildResponse(ParamRequest<T> request, HttpResp resp) throws Exception {
        
    	if (resp.isFileResp()) {
    		// 解析文件响应
	    	ResponseParser<T> parser = new FileResponseParser<>(request.getResponseClass());
	        T response = parser.parse(resp);
	        return response;
        } else {
        	// 解析键值对响应
	    	ResponseParser<T> parser = new ParamStringResponseParser<>(request.getResponseClass());
	        T response = parser.parse(resp);
	        if (!response.needCheckSign()) {
	            return response;
	        }
	
	        // 签名验证
	        verifySignature(request, response);
	        return response;
        }
    }

    /**
     * 执行XML请求
     * @param request
     * @param <T>
     * @return
     */
    private <T extends GcbPayResponse> T executeXmlRequest(XmlRequest<T> request) throws Exception {
        XmlRequestWrapper requestWrapper = request.generateXmlWrapper();
        String xml = XmlUtils.toXml(requestWrapper);
        String signedXml = signXml(xml, request);
        //2022-12-29lcs添加，替换windows下换行符\r\n中的\r
        signedXml = signedXml.replaceAll("\r", "");
        System.out.println("Request: " + signedXml);
        
        HttpResp resp = HttpsClientUtils.postXml(serverUrl, signedXml, GcbPayConstants.ENCODING);
        if (!resp.isSuccess()) {
            T response = request.getResponseClass().newInstance();
            response.setHttpStatus(resp.getHttpStatus());
            response.setResponseBody(resp.getResponseBody());
            return response;
        }

        String respTxt = new String(resp.getResponseBody(), GcbPayConstants.ENCODING);
        // 响应报文验签
        verifyXmlSignature(respTxt);

        Class<?> responseWrapperClass = requestWrapper.getResponseWrapperClass();
        XmlResponseWrapper<T> responseWrapper = (XmlResponseWrapper) XmlUtils.toBean(respTxt, responseWrapperClass);
        T response = responseWrapper.toResponse();
        response.setHttpStatus(resp.getHttpStatus());
        response.setResponseBody(resp.getResponseBody());
        return response;
    }

    /**
     * xml响应报文验签
     * @param responseXml
     */
    private void verifyXmlSignature(String responseXml) throws Exception {
        Document doc = XmlUtils.toDocument(responseXml);
        boolean verifyResult = XMLSecurityUtils.verifySignature(doc, gcbPublicKey);
        if (!verifyResult) {
            throw new GcbPayException("verify response signature failed");
        }
    }

    /**
     * xml加签
     * @param xml
     * @param request
     * @param <T>
     * @return
     * @throws GcbPayException
     */
    private <T extends GcbPayResponse> String signXml(String xml, XmlRequest<T> request) throws Exception {
        Document doc = XmlUtils.toDocument(xml);
        String privateKey = getPrivateKey(request);

        // 加签
        doc = XMLSecurityUtils.xmlSignature(doc, privateKey);
        return XmlUtils.toXml(doc);
    }

    /**
     * 返回签名验证
     * @param request
     * @param response
     * @param <T>
     * @throws GcbPayException
     */
    private <T extends GcbPayResponse> void verifySignature(ParamRequest<T> request, T response) throws GcbPayException {
        ParamResponse paramResponse = (ParamResponse) response;
        Map<String, String> paramMap = paramResponse.getParamMap();
        String sourceMsg = SignUtils.buildSignSourceMsg(paramResponse.getSignParamNames(), paramMap);
        System.out.println("sourceMsg: " + sourceMsg);
        boolean verifyResult = SignUtils.verifyMessage(gcbPublicKey, sourceMsg, paramMap.get(SIGN_MESSAGE_KEY));
        System.out.println("verifyResult: " + verifyResult);
        if (!verifyResult) {
            throw new GcbPayException("verify response signature failed");
        }
    }

    /**
     * 请求加签
     * @param request
     * @param <T>
     * @return
     */
    private <T extends GcbPayResponse> Map<String, String> buildRequest(ParamRequest<T> request) throws Exception {
        Map<String, String> requestData = request.getRequestData();

        // 部分接口不需要签名
        String[] signParamNames = request.getSignParamNames();
        if (signParamNames == null) {
            return requestData;
        }

        // 请求签名
        String privateKeyString = getPrivateKey(request);
        String signSourceMsg = SignUtils.buildSignSourceMsg(signParamNames, requestData);
        String signMsg = SignUtils.signMessage(privateKeyString, signSourceMsg);
        requestData.put(SIGN_MESSAGE_KEY, signMsg);

        if (request instanceof EncryptParamRequest) {
            // 参数加密
            encryptParams(requestData, request);
        }

        return requestData;
    }

    /**
     * 参数加密
     * @param requestData
     * @param request
     * @param <T>
     * @throws Exception
     */
    private <T extends GcbPayResponse> void encryptParams(Map<String, String> requestData, ParamRequest<T> request) throws Exception {
        EncryptParamRequest<T> encryptRequest = (EncryptParamRequest<T>) request;
        String[] encryptParamNames = encryptRequest.getEncryptParamNames();
        if (encryptParamNames == null || encryptParamNames.length == 0) {
            return;
        }

        for (String paramName : encryptParamNames) {
            String paramValue = requestData.get(paramName);
            String encryptValue = SecurityUtils.encrypt(paramValue, gcbPublicKey);
            requestData.put(paramName, encryptValue);
        }
    }

    /**
     * 获取商户私钥
     * @param request
     * @param <T>
     * @return
     * @throws GcbPayException
     */
    private <T extends GcbPayResponse> String getPrivateKey(GcbPayRequest<T> request) throws GcbPayException {
        String merchantId = request.getMerchantId();
        if (configMap.containsKey(merchantId)) {
            return configMap.get(merchantId).getPrivateKey();
        }

        if (StringUtils.isBlank(privateKey)) {
            throw new GcbPayException(merchantId + " privateKey not configured");
        }

        return privateKey;
    }

    public String getServerUrl() {
        return serverUrl;
    }

    public void setServerUrl(String serverUrl) {
        this.serverUrl = serverUrl;
    }

    public String getGcbPublicKey() {
        return gcbPublicKey;
    }

    public void setGcbPublicKey(String gcbPublicKey) {
        this.gcbPublicKey = gcbPublicKey;
    }

    public String getPrivateKey() {
        return privateKey;
    }

    public void setPrivateKey(String privateKey) {
        this.privateKey = privateKey;
    }
}